import csv              # lib to open the csv files


# function for counting lines in a file
def count_rows(filename):
    # get the number of rows
    row_count = sum(1 for row in csv.reader(open(filename)))
    return row_count


# function for replacing columns
def edit_columns(files, changing_indexes, items):
    # get file names
    input_file, output_file = files
    
    # get items lists
    old_items, new_items = items
    
    # open input and output files
    reader = csv.reader(open(input_file, 'rb'))
    writer = csv.writer(open(output_file,'wb'))

    # get the number of columns of the file
    col_num = len(reader.next())

    # read each row from the input file
    for row in reader:
        # get each item from the list of strings to be changed, and index
        for index, old_item in enumerate(old_items):
            # do all the replacements on the selected columns
            for changing_index in changing_indexes:
                new_item = new_items[index]
                replaced = row[changing_index].replace(old_item, new_item)
        # print the replaced field on screen
        print replaced
        
        # building the new row
        new_row = []
        # iterate over all the row fields
        for row_index in range(0, col_num):
            # iterate over all the columns to be changed
            for changing_index in changing_indexes:
                # if the row is not the one to be changed
                if row_index != changing_index:
                    # append the old row field to the new one
                    new_row.append(row[row_index])
                else:
                    # if not append the replaced field
                    new_row.append(replaced)
        
        # write the new row to the new file
        writer.writerow(new_row)


# function for joing csv files
def join_files(filename, limits):
    # get limits
    lower_limit, upper_limit = limits
    
    # create the file where the other files are going to be joined
    writer = csv.writer(open(filename + '.csv', 'a'))
    
    # get header
    top = csv.reader(open(filename + '_' + str(lower_limit).zfill(2) + '.csv'))
    header = next(top)
    
    # write header
    writer.writerow(header)
    
    # iterate over the filename suffixes
    for num in range(lower_limit, upper_limit):
        # read file input
        reader = csv.reader(open(filename + '_' + str(num).zfill(2) + '.csv'))
        
        # skip the header and save it
        next(reader)
        
        # iterate over the rows of file
        for line in reader:
            # write the row in the output file
            writer.writerow(line)
            #print row on the screen
            print line


# function for splitting csv files
def split_file(filename, divisor):
    outfileno = 1           # index for the output files

    # read the file
    reader = csv.reader(open(filename + '.csv', 'r')) 

    # get the header and skip it
    header = next(reader)

    # iterate over the files rows and indexes    
    for index, row in enumerate(reader):
        # everytime the index is a multiple of the divisor
        # create a now file and add the header
        if index % divisor == 0:
            # create output filename
            outfilename = filename + '_' + str(outfileno).zfill(2) + '.csv'
            # print output filename on screen
            print outfilename
            # create output file
            writer = csv.writer(open(outfilename, 'w'))
            # increment index of the output files
            outfileno += 1
            
            # place header
            writer.writerow(header)
        
        # write row
        writer.writerow(row)


# function for deleting columns
def delete_columns(input_file, output_file, columns):
    # open input and output files
    reader = csv.reader(open(input_file, 'rb'))
    writer = csv.writer(open(output_file,'wb'))
    
    # create a new file without the selected rows
    for row in reader:
        for column in columns:
            del row[column]
        writer.writerow(row)


# function for removing duplicates
def remove_duplicates(input_file, output_file, column_id):
    # open input and output files
    reader = csv.reader(open(input_file, 'rb'))
    writer = csv.writer(open(output_file, 'wb'))
    
    # list where iterated items are stored for comparison
    item_id = set()
    for row in reader:
        # if item wasn't iterated yet write row in new file
        # and add id to list of iterated files
        if row[column_id] not in item_id:
            writer.writerow(row)
            item_id.add(row[column_id])
        # else print on screen duplicated id of row
        else:
            print row[column_id]


# main function
if __name__ == "__main__":
    #/////////////////#
    # count_rows test #
    #/////////////////#
    
    # variable definitions
    filename = "epathchina.csv"         # name of the file
    
    # function evocation
    print count_rows(filename)
    
    
    #///////////////////#
    # edit_columns test #
    #///////////////////#
    
    # variable definitions
    input_file = 'epathchina.csv'       # input file
    output_file = 'new_epathchina.csv'  # output file
    changing_indexes = [0]              # index of the column to be replaced
    old_items = [':']                   # list of strings to be replaced
    new_items = ['&']                   # list of replacing strings

    files = [input_file, output_file]   # files list
    items = [old_items, new_items]      # items list
    
    # function evocation
    edit_columns(files, changing_indexes, items)
    
    
    #/////////////////#
    # join_files test #
    #/////////////////#
    
    filename = 'epathchina'                 # name of the file
    lower_limit = 1                         # suffix lower limit
    upper_limit = 14                        # suffix upper limit

    limits = [lower_limit, upper_limit]     # file suffix limits
    
    #function evocation
    join_files(filename, limits)
    
    
    #/////////////////#
    # split_file test #
    #/////////////////#
    
    divisor = 1000              # number of rows the file is going to be broken
    filename = 'epathchina'     # name suffix for the files
    
    #function evocation
    split_file(filename, divisor)
    
    
    #/////////////////////#
    # delete_columns test #
    #/////////////////////#
    
    input_file = 'epathchina.csv'       # input file
    output_file = 'new_epathchina.csv'  # output file
    
    columns = [1, 3]                    # columns to be deleted
    
    #function evocation
    delete_columns(files, columns)
    
    
    #////////////////////////#
    # remove_duplicates test #
    #////////////////////////#    
    
    input_file = 'epathchina.csv'       # input file
    output_file = 'new_epathchina.csv'  # output file
    
    column_id = 2                       # columns to be deleted
    
    #function evocation    
    remove_duplicates(input_file, output_file, column_id)
    
    
